# $Id: mt-uploaddir.pl 51 2015-10-07 08:28:43Z tajima $

package MT::Plugin::UploadDir;

use strict;
use MT::Plugin;
@MT::Plugin::UploadDir::ISA = qw(MT::Plugin);

my $DEFAULT_EXT = "audio:mp3,wma,m4a,midi,wav,aiff\n";
$DEFAULT_EXT .= "videos:mp4,m4v,mpeg,avi,mov,wmv\n";
$DEFAULT_EXT .= "images:bmp,jpg,jpeg,gif,tif,tiff,png\n";
$DEFAULT_EXT .= "text:txt\n";
$DEFAULT_EXT .= "docs:pdf,doc,xls,ppt\n";
$DEFAULT_EXT .= "src:pl,c,cc,pas,rb\n";
$DEFAULT_EXT .= "archive:bz2,cab,gz,jar,lzh,rar,tar,taz,zip\n";

use vars qw($PLUGIN_NAME $VERSION);
$PLUGIN_NAME = 'UploadDir';
$VERSION = '0.70';

use MT;
use MT::Util;
my $plugin_registries = MT->version_number >= 6.2 ? {
    schema_version => '1.0',
    registry => {
        object_types => {
            'blog' => {
                'enable_uploaddir' => 'integer meta',
            },
        },
    },
} : {};
my $plugin = new MT::Plugin::UploadDir({
    name => $PLUGIN_NAME,
    version => $VERSION,
    description => "<MT_TRANS phrase='This plugin automatically switches the destination directory for the uploaded file by the file extension.'>",
    doc_link => 'http://labs.m-logic.jp/plugins/mt-uploaddir/docs/'.$VERSION.'/mt-uploaddir.html',
    author_name => 'M-Logic, Inc.',
    author_link => 'http://labs.m-logic.jp/',
    blog_config_template => \&template,
    l10n_class => 'UploadDir::L10N',
    settings => new MT::PluginSettings([
        ['upload_dir_ext_list', { Default => $DEFAULT_EXT }],
        ['upload_dir_ext_default']
    ]),
    %$plugin_registries,
});

MT->add_plugin($plugin);
if (MT->version_number < 3) {
    MT->add_callback('MT::App::CMS::AppTemplateSource.upload', 9, $plugin, \&hdlr_source_upload);
}
if (MT->version_number >= 4) {
    # for MT4+
    MT->add_callback('MT::App::CMS::template_source.asset_upload', 9, $plugin, \&hdlr_source_asset_upload);
}
if (MT->version_number >= 6.2) {
    MT->add_callback('MT::App::CMS::template_source.async_asset_upload', 9, $plugin, \&hdlr_source_async_asset_upload);
    MT->add_callback('MT::App::CMS::template_param.asset_upload', 9, $plugin, \&hdlr_param_cfg_prefs);
    MT->add_callback('MT::App::CMS::template_source.cfg_prefs', 9, $plugin, \&hdlr_source_cfg_prefs);
    MT->add_callback('MT::App::CMS::template_param.cfg_prefs', 9, $plugin, \&hdlr_param_cfg_prefs);
    MT->add_callback('MT::App::CMS::cms_pre_save.blog', 9, $plugin, \&hdlr_cms_pre_save_blog);
}

sub instance { $plugin; }

sub template {
    my $tmpl = <<'EOT';
    <div class="setting">
    <div class="label">
    <label for="upload_dir_ext_list"><MT_TRANS phrase="Extensions:"></label>
    </div>
    <div class="field">
    <p><textarea name="upload_dir_ext_list" id="upload_dir_ext_list" cols="60" rows="10"><TMPL_VAR NAME=UPLOAD_DIR_EXT_LIST ESCAPE=HTML></textarea></p>
    </div> 
    </div>
    <div class="setting">
    <div class="label">
    <label for="upload_dir_ext_default"><MT_TRANS phrase="Default directory:"></label>
    </div>
    <div class="field">
    <p><input type="text" name="upload_dir_ext_default" id="upload_dir_ext_default" value="<TMPL_VAR NAME=UPLOAD_DIR_EXT_DEFAULT ESCAPE=HTML>" /></p>
    </div> 
    </div>
EOT
}

sub upload_dir_ext_list {
    my $plugin = shift;
    my ($blog_id) = @_;
    my %plugin_param;

    $plugin->load_config(\%plugin_param, 'blog:'.$blog_id);
    my $key = $plugin_param{upload_dir_ext_list};
    unless ($key) {
        $plugin->load_config(\%plugin_param, 'system');
        $key = $plugin_param{upload_dir_ext_list};
    }
    $key;
}

sub upload_dir_ext_default {
    my $plugin = shift;
    my ($blog_id) = @_;
    my %plugin_param;

    $plugin->load_config(\%plugin_param, 'blog:'.$blog_id);
    my $key = $plugin_param{upload_dir_ext_default};
    unless ($key) {
        $plugin->load_config(\%plugin_param, 'system');
        $key = $plugin_param{upload_dir_ext_default};
    }
    $key;
}

sub parse_ext {
    my $plugin = shift;
    my $ext_list = shift;

    $ext_list =~ s/,/\|/g;

    my @lines = split(/\n/, $ext_list);

    my %ext = ();
    foreach my $line (@lines) {
        $line =~ s/[\r\n]//g;
        last if !$line;
        $line =~ m/([^:]+):(.+)$/;
        $ext{$2} = $1;
    }

    %ext;
}

sub make_script {
    my ($blog_id) = @_;

    my $ext_list = $plugin->upload_dir_ext_list($blog_id);
    my $ext_default = $plugin->upload_dir_ext_default($blog_id);

    my %ext = $plugin->parse_ext($ext_list);

    my $ext_if;
    foreach my $key ( keys %ext ) {
        $ext_if .= 'if (fln.match(/\.(' . $key . ')$/i)){' . "\n";
        $ext_if .= '    el.value = \'' . $ext{$key} . "'\n";
        $ext_if .= '} else '
    }
    $ext_if .= "{ \n";
    $ext_if .= "    el.value = '" . $ext_default . "';\n";
    $ext_if .= "}\n";

    my $script = <<"HTML";
<script type="text/javascript" language="JavaScript">
<!--
function changePath(fl){
var fln = fl.value;
var el = document.getElementById('extra_path');

$ext_if
}
//-->
</script>
HTML

    $script;
}

sub make_setting_tmpl {
    my $app = shift;
    my $blog = $app->blog;
    my $blog_id = $blog->id;

    my $label = $plugin->translate('Determined by extensions');
    my $display = $plugin->translate('Show configuration');
    my $hide = $plugin->translate('Hide configuration');
    my $ext_list = $plugin->upload_dir_ext_list($blog_id);
    my $ext_default = $plugin->upload_dir_ext_default($blog_id);
    my %ext = $plugin->parse_ext($ext_list);
    my $table = '<thead>';
    $table .= '<tr><th class="dir">' . $plugin->translate('Directory') . '</th><th class="ext">' . $plugin->translate('Extensions') . "</th></tr>\n";
    foreach my $key ( keys %ext ) {
        $table .= '<tr><td class="dir">' . MT::Util::encode_html($ext{$key}) . '</td>';
        $key =~ s/\|/, /g;
        $table .= '<td class="ext">' . MT::Util::encode_html($key) . "</td></tr>\n";
    }
    $table .= '<tr><td class="dir default">' . MT::Util::encode_html($ext_default) . '</td><td class="ext default">' . $plugin->translate('Default') . "</td></tr>\n";
    $table .= "</tbody>\n";
    my $script = <<"HTML";
<style>
#uploaddir_extlist {
    display: block;
    border-radius: 3px;
    border: 1px solid #c0c6c9;
    border-collapse: collapse;
}
#uploaddir_extlist th {
    text-align: left;
    background-color: #e6e6e6;
    border-bottom: 1px solid #c0c6c9;
}
#uploaddir_extlist th,
#uploaddir_extlist td {
    margin: 0;
    padding: 3px;
}
#uploaddir_extlist .dir {
    border-right: 1px solid #c0c6c9;
}
#uploaddir_extlist .default {
    border-top: 1px dotted #c0c6c9;
}
</style>
<div style="display:inline-block;vertical-align: text-top;">
<input type="checkbox" name="enable_uploaddir" id="enable_uploaddir" value="1" class="cb" />
<label for="enable_uploaddir">$label</label>&nbsp;<span>(<a href="javascript:void(0)" class="toggle_uploaddir">$display</a>)</span>
<table id="uploaddir_extlist" class="hidden">
$table
</table>
</div>
<mt:setvarblock name="jq_js_include" append="1">
jQuery('#enable_uploaddir').change( function() {
  if (jQuery(this).prop('checked')) {
    jQuery('#extra_path').addClass('disabled').attr('disabled', 'disabled');
  }
  else {
    jQuery('#extra_path').removeClass('disabled').removeAttr('disabled');
  }
});
<mt:if name="enable_uploaddir">
jQuery(function() {
  jQuery('#enable_uploaddir').prop('checked', true);
  jQuery('#extra_path').addClass('disabled').attr('disabled', 'disabled');
});
</mt:if>
jQuery('.toggle_uploaddir').click(function() {
  var list=jQuery('#uploaddir_extlist');
  if (list.hasClass('hidden')) {
    jQuery(this).text('$hide');
    list.removeClass('hidden');
  }
  else {
    jQuery(this).text('$display');
    list.addClass('hidden');
  }
  return false;
});
</mt:setvarblock>
HTML

    $script;
}

sub hdlr_source_upload {
    my ($eh, $app, $tmpl_ref) = @_;
    my $blog = $app->blog;
    if (MT->version_number < 3.4) {
        # hdlr_source_3_3(@_);
        my $old = <<'HTML';
<p><label for="file"><MT_TRANS phrase="File:"></label> <input type="file" name="file" /></p>
HTML
        my $script = make_script($blog->id);
        my $new = <<"HTML";
$script

<p><label for="file"><MT_TRANS phrase="File:"></label> <input type="file" name="file" onchange="changePath(this)" /></p>
HTML
        $$tmpl_ref =~ s!$old!$new!;
    } else {
        # hdlr_source_w(@_);
        my $old = <<'HTML';
<p><input type="file" name="file" /></p>
HTML
        my $script = make_script($blog->id);
        my $new = <<"HTML";
$script

<p><input type="file" name="file" onchange="changePath(this)" /></p>
HTML
       $$tmpl_ref =~ s!$old!$new!;
    }
    1;
}

sub hdlr_source_asset_upload {
    my ($eh, $app, $tmpl_ref) = @_;
    my $blog = $app->blog;
    if (MT->version_number < 4.2) {
        # MT4
        my $old = <<'HTML';
<input type="file" name="file" />
HTML
        my $script = make_script($blog->id);
        my $new = <<"HTML";
$script

<input type="file" name="file" onchange="changePath(this)" />
HTML
        $$tmpl_ref =~ s!$old!$new!;
    } else {
        # MT4.2+
        my $old = <<'HTML';
<input type="file" name="file" id="file" />
HTML
        my $script = make_script($blog->id);
        my $new = <<"HTML";
$script

<input type="file" name="file" id="file" onchange="changePath(this)" />
HTML
        $$tmpl_ref =~ s!$old!$new!;
    }
    1;
}

sub hdlr_source_async_asset_upload {
    my ($eh, $app, $tmpl_ref) = @_;
    my $blog = $app->blog;
    my $blog_id = $blog->id;
    my $old = quotemeta('<input type="text" name="extra_path" id="extra_path" class="text path" value="<mt:var name="extra_path" escape="html">" />');
    my $new = make_setting_tmpl($app);
    $$tmpl_ref =~ s!($old)!$1\n$new!;
    $old = quotemeta('<input type="hidden" id="extra_path" name="extra_path" value="<mt:var name="extra_path" escape="html">" />');
    $new = <<'HTML';
<input type="hidden" id="enable_uploaddir" name="enable_uploaddir" value="<mt:var name="enable_uploaddir" escape="html">" />
HTML
    $$tmpl_ref =~ s!($old)!$1\n$new!;
    my $label = $plugin->translate('Determined by extensions');
    $old = quotemeta('<mt:var name="upload_destination_label">/<mt:var name="extra_path">');
    $new = <<"HTML";
<mt:var name="upload_destination_label">/<mt:if name="enable_uploaddir"> ($label)<mt:else><mt:var name="extra_path"></mt:if>
HTML
    $$tmpl_ref =~ s!$old!$new!;

    my $ext_list = $plugin->upload_dir_ext_list($blog_id);
    my $ext_default = $plugin->upload_dir_ext_default($blog_id);
    my %ext = $plugin->parse_ext($ext_list);
    my $ext_if = '';
    foreach my $key ( keys %ext ) {
        $ext_if .= 'if (fl.match(/\.(' . $key . ')$/i)){' . "\n";
        $ext_if .= '    dn = \'' . $ext{$key} . "';\n";
        $ext_if .= '} else '
    }
    $ext_if .= "{ \n";
    $ext_if .= "    dn = '" . $ext_default . "';\n";
    $ext_if .= "}\n";
    $new = <<"HTML";
console.log(jQuery('#enable_uploaddir').val());
if (jQuery('#enable_uploaddir').val()) {
var fl = file.name;
var dn = '';
$ext_if
fd.append('extra_path', dn );
}
else {
    fd.append('extra_path', jQuery('#extra_path').val() );
}
HTML
    $old = quotemeta('fd.append(\'extra_path\', jQuery(\'#extra_path\').val() );');
    $$tmpl_ref =~ s!$old!$new!;
    1;
}

sub hdlr_source_cfg_prefs {
    my ($eh, $app, $tmpl_ref) = @_;
    my $old = quotemeta('<input type="text" name="extra_path" id="extra_path" class="text path" value="<mt:var name="extra_path" escape="html">" />');
    my $new = make_setting_tmpl($app);
    $$tmpl_ref =~ s!($old)!$1\n$new!;
    1;
}

sub hdlr_param_cfg_prefs {
    my ( $cb, $app, $param, $tmpl ) = @_;
    my $blog = $app->blog || return;
    $param->{enable_uploaddir} = $blog->enable_uploaddir;
}

sub hdlr_cms_pre_save_blog {
    my ($cb, $app, $obj, $original) = @_;
    my $enabled = $app->param('enable_uploaddir');
    $obj->enable_uploaddir($enabled ? 1 : 0);
    return 1;
}

1;
